
From: Keshavamurthy Anil S <anil.s.keshavamurthy@intel.com>

Not safe to insert kprobes on IVT code.

This patch checks to see if the address on which Kprobes is being inserted is
in ivt code and if it is in ivt code then refuse to register kprobe.

Signed-off-by: Anil S Keshavamurthy <anil.s.keshavamurthy@intel.com>
Acked-by: David Mosberger <davidm@napali.hpl.hp.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 arch/ia64/kernel/kprobes.c     |   18 ++++++++++++++++--
 arch/ia64/kernel/vmlinux.lds.S |    7 ++++++-
 include/asm-ia64/sections.h    |    1 +
 3 files changed, 23 insertions(+), 3 deletions(-)

diff -puN arch/ia64/kernel/kprobes.c~kprobes-ia64-refuse-kprobe-on-ivt-code arch/ia64/kernel/kprobes.c
--- 25/arch/ia64/kernel/kprobes.c~kprobes-ia64-refuse-kprobe-on-ivt-code	2005-06-24 23:24:01.000000000 -0700
+++ 25-akpm/arch/ia64/kernel/kprobes.c	2005-06-24 23:24:48.000000000 -0700
@@ -34,6 +34,7 @@
 
 #include <asm/pgtable.h>
 #include <asm/kdebug.h>
+#include <asm/sections.h>
 
 extern void jprobe_inst_return(void);
 
@@ -263,14 +264,27 @@ static inline void get_kprobe_inst(bundl
 	}
 }
 
+/* Returns non-zero if the addr is in the Interrupt Vector Table */
+static inline int in_ivt_functions(unsigned long addr)
+{
+	return (addr >= (unsigned long)__start_ivt_text
+		&& addr < (unsigned long)__end_ivt_text);
+}
+
 static int valid_kprobe_addr(int template, int slot, unsigned long addr)
 {
 	if ((slot > 2) || ((bundle_encoding[template][1] == L) && slot > 1)) {
-		printk(KERN_WARNING "Attempting to insert unaligned kprobe at 0x%lx\n",
-				addr);
+		printk(KERN_WARNING "Attempting to insert unaligned kprobe "
+				"at 0x%lx\n", addr);
 		return -EINVAL;
 	}
 
+ 	if (in_ivt_functions(addr)) {
+ 		printk(KERN_WARNING "Kprobes can't be inserted inside "
+				"IVT functions at 0x%lx\n", addr);
+ 		return -EINVAL;
+ 	}
+
 	if (slot == 1 && bundle_encoding[template][1] != L) {
 		printk(KERN_WARNING "Inserting kprobes on slot #1 "
 		       "is not supported\n");
diff -puN arch/ia64/kernel/vmlinux.lds.S~kprobes-ia64-refuse-kprobe-on-ivt-code arch/ia64/kernel/vmlinux.lds.S
--- 25/arch/ia64/kernel/vmlinux.lds.S~kprobes-ia64-refuse-kprobe-on-ivt-code	2005-06-24 23:24:01.000000000 -0700
+++ 25-akpm/arch/ia64/kernel/vmlinux.lds.S	2005-06-24 23:24:01.000000000 -0700
@@ -8,6 +8,11 @@
 #define LOAD_OFFSET	(KERNEL_START - KERNEL_TR_PAGE_SIZE)
 #include <asm-generic/vmlinux.lds.h>
 
+#define IVT_TEXT							\
+		VMLINUX_SYMBOL(__start_ivt_text) = .;			\
+		*(.text.ivt)						\
+		VMLINUX_SYMBOL(__end_ivt_text) = .;
+
 OUTPUT_FORMAT("elf64-ia64-little")
 OUTPUT_ARCH(ia64)
 ENTRY(phys_start)
@@ -39,7 +44,7 @@ SECTIONS
 
   .text : AT(ADDR(.text) - LOAD_OFFSET)
     {
-	*(.text.ivt)
+	IVT_TEXT
 	*(.text)
 	SCHED_TEXT
 	LOCK_TEXT
diff -puN include/asm-ia64/sections.h~kprobes-ia64-refuse-kprobe-on-ivt-code include/asm-ia64/sections.h
--- 25/include/asm-ia64/sections.h~kprobes-ia64-refuse-kprobe-on-ivt-code	2005-06-24 23:24:01.000000000 -0700
+++ 25-akpm/include/asm-ia64/sections.h	2005-06-24 23:24:01.000000000 -0700
@@ -17,6 +17,7 @@ extern char __start_gate_vtop_patchlist[
 extern char __start_gate_fsyscall_patchlist[], __end_gate_fsyscall_patchlist[];
 extern char __start_gate_brl_fsys_bubble_down_patchlist[], __end_gate_brl_fsys_bubble_down_patchlist[];
 extern char __start_unwind[], __end_unwind[];
+extern char __start_ivt_text[], __end_ivt_text[];
 
 #endif /* _ASM_IA64_SECTIONS_H */
 
_
